@@ -1,6 +1,6 @@
 ;;; ebdb-format.el --- Formatting/exporting EBDB records  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2016-2017  Free Software Foundation, Inc.
+;; Copyright (C) 2016-2018  Free Software Foundation, Inc.
 
 ;; Author: Eric Abrahamsen <eric@ericabrahamsen.net>
 
@@ -35,8 +35,12 @@
 (defvar ebdb-formatter-tracker nil
   "Variable for holding all instantiated formatters.")
 
-(defclass ebdb-formatter (eieio-named eieio-instance-tracker)
-  ((tracking-symbol :initform ebdb-formatter-tracker)
+(defclass ebdb-formatter (eieio-instance-tracker)
+  ((label
+    :initarg :label
+    :type string
+    :initform "")
+   (tracking-symbol :initform ebdb-formatter-tracker)
    (coding-system
     :type symbol
     :initarg :coding-system
@@ -44,30 +48,29 @@
     :initform `,buffer-file-coding-system
     :documentation "The coding system for the formatted
     file/buffer/stream.")
-   ;; TODO: Provide for "psuedo field classes" like 'primary-mail and
-   ;; 'role-mail.
+   ;; The elements of the next two slots, besides field class symbols,
+   ;; can also use some shortcut symbols: mail, phone, address, notes,
+   ;; tags, role, mail-primary, mail-defunct, mail-not-defunct,
+   ;; role-defunct, and role-not-defunct.
    (include
     :type list
     :initarg :include
     :initform nil
-    :documentation "A list of field classes to include.  If
-    \"include\" and \"exclude\" conflict, \"exclude\" loses.")
+    :documentation "A list of field classes to include.")
    (exclude
     :type list
     :initarg :exclude
-    :initform '(ebdb-field-uuid ebdb-field-timestamp ebdb-field-creation-date)
-    :documentation "A list of field classes to exclude.")
+    :initform nil
+    :documentation "A list of field classes to exclude.  This
+    slot is only honored if \"include\" is nil.")
    (sort
     :type list
     :initarg :sort
-    :initform '(ebdb-field-mail ebdb-field-phone ebdb-field-address "_" ebdb-field-notes)
+    :initform '(ebdb-field-mail
+		ebdb-field-phone ebdb-field-address "_" ebdb-field-notes)
     :documentation "How field instances should be sorted.  Field
     classes should be listed in their proper sort order.  A \"_\"
     placeholder indicates where all other fields should go." )
-   (primary
-    :type boolean
-    :initarg :primary
-    :initform nil)
    (header
     :type list
     :initarg :header
@@ -78,13 +81,13 @@
    (combine
     :type list
     :initarg :combine
-    :initform '(ebdb-field-mail ebdb-field-phone)
+    :initform nil
     :documentation "A list of field classes which should be
     output with all instances grouped together.")
    (collapse
     :type list
     :initarg :collapse
-    :initform '(ebdb-field-address)
+    :initform nil
     :documentation "A list of field classes which should be
     \"collapsed\". What this means is up to the formatter, but it
     generally indicates that most of the field contents will
@@ -101,7 +104,7 @@
   Subclass this to produce real formatters.")
 
 (cl-defmethod ebdb-string ((fmt ebdb-formatter))
-  (slot-value fmt 'object-name))
+  (slot-value fmt 'label))
 
 (cl-defgeneric ebdb-fmt-header (fmt records)
   "Insert a string at the beginning of the list of records.")
@@ -204,7 +207,7 @@ FIELD-STRING1 FIELD-STRING2 ..)."
 				    (field ebdb-field-labeled)
 				    _style
 				    (_record ebdb-record))
-  (eieio-object-name-string field))
+  (ebdb-field-label field))
 
 (cl-defmethod ebdb-fmt-field-label ((_fmt ebdb-formatter)
 				    (field ebdb-field-labeled)
@@ -217,7 +220,7 @@ FIELD-STRING1 FIELD-STRING2 ..)."
 			      (_style (eql compact))
 			      (record ebdb-record))
   (format "(%s) %s"
-	  (eieio-object-name-string field)
+	  (ebdb-field-label field)
 	  (ebdb-fmt-field fmt field 'oneline record)))
 
 (cl-defmethod ebdb-fmt-field ((_fmt ebdb-formatter)
@@ -256,15 +259,13 @@ FIELD-STRING1 FIELD-STRING2 ..)."
 			  ;; be removed at some point.
 			  (lambda (elt) (or (eql (car elt) 'name)
 					    (null (cdr elt))))
- 			  (ebdb-record-current-fields record nil t)))))
-	f-class)
+ 			  (ebdb-record-current-fields record nil t))))))
     (with-slots (exclude include) fmt
       (seq-filter
        (lambda (f)
-	 (setq f-class (eieio-object-class-name f))
 	 (if include
-	     (ebdb-class-in-list-p f-class include)
-	   (null (ebdb-class-in-list-p f-class exclude))))
+	     (ebdb-foo-in-list-p f include)
+	   (null (ebdb-foo-in-list-p f exclude))))
        fields))))
 
 (cl-defmethod ebdb-fmt-collect-fields ((fmt ebdb-formatter)
@@ -277,63 +278,57 @@ FIELD-STRING1 FIELD-STRING2 ..)."
 (cl-defmethod ebdb-fmt-sort-fields ((fmt ebdb-formatter)
 				    (_record ebdb-record)
 				    field-list)
-  (let ((sort (slot-value fmt 'sort))
-	f acc outlist class)
-    (when sort
-      (dolist (s sort)
-	(if (symbolp s)
-	    (progn
-	      (setq class (cl--find-class s))
-	      (while (setq f (pop field-list))
-		(if (same-class-p f class)
-		    (push f outlist)
-		  (push f acc)))
-	      (setq field-list acc
-		    acc nil))
-	  ;; We assume this is the "_" value.  Actually, anything
-	  ;; would do as a catchall placeholder.
-	  (dolist (fld field-list)
-	    (setq class (eieio-object-class-name fld))
-	    (unless (memq class sort)
-	      ;; This isn't enough -- field still need to be grouped
-	      ;; by field class.
-	      (push fld outlist)))))
-      (setq field-list (nreverse outlist)))
-    field-list))
+  "Sort FIELD-LIST using sort order from FMT.
+First sorts all fields with `ebdb-field-compare', then sorts
+again by the order of each field's class symbol in the 'sort
+slot of FMT."
+  (let* ((sort-order (slot-value fmt 'sort))
+	 (catchall (or (seq-position sort-order "_")
+		       (length sort-order)))
+	 (sorted (seq-sort #'ebdb-field-compare field-list)))
+
+    (when sort-order
+      (setq sorted
+	    (seq-sort-by
+	     (lambda (f)
+	       (or (seq-position sort-order (eieio-object-class-name f))
+		   catchall))
+	     #'< sorted)))
+    sorted))
 
 (cl-defmethod ebdb-fmt-process-fields ((fmt ebdb-formatter)
 				       (_record ebdb-record)
 				       field-list)
   "Process FIELD-LIST for FMT.
-
 At present that means handling the combine and collapse slots of
 FMT.
 
 This method assumes that fields in FIELD-LIST have already been
 grouped by field class."
-  (let (outlist cls f acc)
+  (let (outlist f acc)
     (with-slots (combine collapse) fmt
       (when combine
 	(while (setq f (pop field-list))
-	  (setq cls (eieio-object-class-name f))
-	  (if (null (ebdb-class-in-list-p cls combine))
+	  (if (null (ebdb-foo-in-list-p f combine))
 	      (push f outlist)
 	    (push f acc)
-	    (while (and field-list (same-class-p (car field-list) (eieio-object-class f)))
+	    (while (and field-list (same-class-p (car field-list)
+						 (eieio-object-class f)))
 	      (push (setq f (pop field-list)) acc))
-	    (push `(:class ,cls :style compact :inst ,acc) outlist)
+	    (push `(:class ,(eieio-object-class-name f)
+			   :style compact :inst ,(nreverse acc))
+		  outlist)
 	    (setq acc nil)))
 	(setq field-list (nreverse outlist)
 	      outlist nil))
       (dolist (f field-list)
 	(if (listp f)
 	    (push f outlist)
-	  (setq cls (eieio-object-class-name f))
-	  (push (list :class cls
+	  (push (list :class (eieio-object-class-name f)
 		      :inst (list f)
 		      :style
 		      (cond
-		       ((ebdb-class-in-list-p cls collapse) 'collapse)
+		       ((ebdb-foo-in-list-p f collapse) 'collapse)
 		       (t 'normal)))
 		outlist)))
       outlist)))
@@ -350,7 +345,7 @@ grouped by field class."
   (let ((collection
 	 (mapcar
 	  (lambda (formatter)
-	    (cons (slot-value formatter 'object-name) formatter))
+	    (cons (slot-value formatter 'label) formatter))
 	  ebdb-formatter-tracker)))
     (cdr (assoc (completing-read "Use formatter: " collection)
 		collection))))
