@@ -1,6 +1,6 @@
 ;;; ebdb-mua.el --- Mail user agent interaction for EBDB  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2016-2017  Free Software Foundation, Inc.
+;; Copyright (C) 2016-2018  Free Software Foundation, Inc.
 
 ;; Author: Eric Abrahamsen <eric@ericabrahamsen.net>
 
@@ -69,6 +69,10 @@
   "For communication between `ebdb-update-records' and `ebdb-query-create'.
 It is a list with elements (NAME MAIL HEADER HEADER-CLASS MUA).")
 
+(defgroup ebdb-mua nil
+  "Variables that specify the EBDB-MUA interface"
+  :group 'ebdb)
+
 (defcustom ebdb-mua-auto-update-p 'existing
   "Specify how EBDB handles addresses in mail messages.
 It can take one of the following values:
@@ -223,7 +227,7 @@ accordingly."
 	((eq ebdb-user-mail-address-re 'self)
 	 (let ((self-rec (ebdb-record-self)))
 	   (unless self-rec
-	     (user-error "`ebdb-user-mail-address-re' set to 'self, but `ebdb-record-self' is not set.? "))
+	     (user-error "`ebdb-user-mail-address-re' set to 'self, but `ebdb-record-self' is not set"))
 	   (setq ebdb-user-mail-address-re
 		 (regexp-opt (slot-value
 			      (ebdb-record-cache self-rec)
@@ -532,8 +536,7 @@ variable should be set before EBDB is loaded.")
     (setq slots (plist-put slots :folder str)))
   (cl-call-next-method c str slots))
 
-(cl-defmethod ebdb-init-field ((f ebdb-field-mail-folder)
-			       &optional record)
+(cl-defmethod ebdb-init-field ((f ebdb-field-mail-folder) record)
   (when record
     (let* ((folder (slot-value f 'folder))
 	   (mails (mapcar #'regexp-quote (ebdb-record-mail-canon record)))
@@ -547,8 +550,8 @@ variable should be set before EBDB is loaded.")
 		ebdb-mail-folder-list)))))
   (cl-call-next-method))
 
-(cl-defmethod ebdb-delete-field ((f ebdb-field-mail-folder)
-				 &optional record _unload)
+(cl-defmethod ebdb-delete-field ((f ebdb-field-mail-folder) record
+				 &optional _unload)
   (when record
     (let* ((folder (slot-value f 'folder))
 	   (mails (mapcar #'regexp-quote (ebdb-record-mail-canon record)))
@@ -595,13 +598,14 @@ HEADER-TYPE, as well as the IGNORE-ADDRESS argument, and the
 variables `ebdb-user-mail-address-re',
 `ebdb-accept-header-alist', and `ebdb-ignore-header-alist'."
   (let ((name (car address-parts))
-	(mail (cadr address-parts)))
+	(mail (cadr address-parts))
+	(user-mail (ebdb-get-user-mail-address-re)))
     (and (null (or (and (stringp ignore-address)
 			(or (and name (string-match-p ignore-address name))
 			    (and mail (string-match-p ignore-address mail))))
-		   (string-match-p
-		    (ebdb-get-user-mail-address-re)
-		    mail)))
+		   (and (stringp user-mail)
+			(null (string-empty-p user-mail))
+			(string-match-p user-mail mail))))
 	 (cond ((null (or ebdb-accept-header-alist
 			  ebdb-ignore-header-alist))
 		t)
@@ -994,8 +998,10 @@ Return the records matching ADDRESS or nil."
                (unless ebdb-silent
                  (if (ebdb-record-name record)
                      (message "created %s's record with address \"%s\""
-                              (ebdb-string record) mail)
-                   (message "created record with naked address \"%s\"" mail)))
+                              (ebdb-string record)
+			      (ebdb-string mail))
+                   (message "created record with naked address \"%s\""
+			    (ebdb-string mail))))
                (ebdb-init-record record))
 
               (change-p
@@ -1004,9 +1010,11 @@ Return the records matching ADDRESS or nil."
                         (message "noticed \"%s\"" (ebdb-string record)))
                        ((ebdb-record-name record)
                         (message "noticed %s's address \"%s\""
-                                 (ebdb-string record) mail))
+                                 (ebdb-string record)
+				 (ebdb-string mail)))
                        (t
-                        (message "noticed naked address \"%s\"" mail))))))
+                        (message "noticed naked address \"%s\""
+				 (ebdb-string mail)))))))
 
         (run-hook-with-args 'ebdb-notice-mail-hook record)
 
@@ -1162,12 +1170,13 @@ where it was in the MUA, rather than quitting the EBDB buffer."
 		  'existing))
 	notes)
     (when records
-      (ebdb-with-record-edits (record records)
-	(setq notes (ebdb-record-field record 'notes))
-	(if notes
-	    (ebdb-record-change-field record notes)
-	  (setq notes (ebdb-read ebdb-default-notes-class))
-	  (ebdb-record-insert-field record notes)))
+      (dolist (record records)
+       (ebdb-with-record-edits record
+	 (setq notes (ebdb-record-field record 'notes))
+	 (if notes
+	     (ebdb-record-change-field record notes)
+	   (setq notes (ebdb-read ebdb-default-notes-class))
+	   (ebdb-record-insert-field record notes))))
       (ebdb-redisplay-records records 'reformat t))))
 
 ;;;###autoload
