@@ -219,11 +219,11 @@ If it doesn't exist, raise `ebdb-related-unfound'."
 	  (ebdb-db-add-record db2 rec1)
 	  (ebdb-db-add-record db1 rec2)
 	  (setf (slot-value db2 'read-only) t)
-	  (ebdb-with-record-edits (r (list rec1 rec2))
-	    (ebdb-record-insert-field
-	     r (ebdb-parse 'ebdb-field-mail "none@such.com")))
-	  ;; rec1 should have been excluded from the list of editable
-	  ;; records, but no error should be raised.
+	  (dolist (rec (list rec1 rec2))
+	    (ebdb-with-record-edits rec
+	      (ebdb-record-insert-field
+	       rec (ebdb-parse 'ebdb-field-mail "none@such.com"))))
+	  ;; Field insertion should have silently failed for rec1.
 	  (should-not
 	   (slot-value rec1 'mail)))))))
 
@@ -347,7 +347,37 @@ If it doesn't exist, raise `ebdb-related-unfound'."
 	   (slot-value
 	    (ebdb-parse 'ebdb-field-name-complex "Eric Abrahamsen, III")
 	    'suffix)
-	   "III")))
+	   "III"))
+  (should (equal
+	   (slot-value
+	    (ebdb-parse 'ebdb-field-name-complex "Albus Percival Wulfric Brian Dumbledore")
+	    'given-names)
+	   '("Albus" "Percival" "Wulfric" "Brian")))
+  (should (equal
+	   (slot-value
+	    (ebdb-parse 'ebdb-field-name-complex "MURAKAMI Haruki")
+	    'surname)
+	   "Murakami"))
+  (should (equal
+	   (slot-value
+	    (ebdb-parse 'ebdb-field-name-complex "Fintan O'Toole")
+	    'surname)
+	   "O'Toole"))
+  (should (equal
+	   (slot-value
+	    (ebdb-parse 'ebdb-field-name-complex "O'Toole, Fintan")
+	    'surname)
+	   "O'Toole"))
+  (should (equal
+	   (slot-value
+	    (ebdb-parse 'ebdb-field-name-complex "O'TOOLE Fintan")
+	    'surname)
+	   "O'Toole"))
+  (should (equal
+	   (slot-value
+	    (ebdb-parse 'ebdb-field-name-complex "Daniel Micahel Blake Day-Lewis")
+	    'surname)
+	   "Day-Lewis")))
 
 ;; Snarf testing.
 
@@ -612,7 +642,7 @@ and typing."))
 			      :mail (list (ebdb-field-mail
 					   :mail "jb@barleycorn.com"))
 			      :phone (list (ebdb-field-phone
-					    :object-name "home"
+					    :label "home"
 					    :country-code 1
 					    :area-code 555
 					    :number "5555555"))
